@@ -1,24 +1,34 @@
 subroutine gradient(pars,ED,Error,npars,iter,&
                     nED,grad,value,errorflag)
 !--------------------------------------------------------------------------------------
-! Subroutine gradient is used to calculate the gradients of
+! Subroutine gradient() is used to calculate the gradients of
 ! specified parameters that belong to a function of interest
 ! while this subroutine constraints the function to be fun34
+! =====================================================================================
 !
-! npars,         input:: integer, the length of the parameters to be analyzed
-! nED,           input:: integer, the length of ED or Error
-! iter,          input:: integer,  the number of Richardson improvement iterations
-! errorflag,    output:: integer, if both value and gradient can be calculated,
-!                        return 0, otherwise 1
-! pars(npars),   input:: real values, the parameters to be analyzed
-! ED(nED),       input:: real values, the equivalent dose
-! Error(nED),    input:: real values, the standard errors of equivalent dose
-! grad(npars),  output:: real values, the calculated gradient
-! value,        output:: real value, the caluculated value for the specified function
+! npars,         input:: integer, the length of the parameters to be analyzed/
 !
-! Author:: Peng Jun, 2013.04.01, revised in 2013.04.21
+! nED,           input:: integer, the length of equivalent doses.
 !
-! Dependence:: inner function fun34; subroutine pnorm; function alnorm
+! iter,          input:: integer,  the number of Richardson improvement iterations.
+!
+! pars(npars),   input:: real values, the parameters to be analyzed.
+!
+! ED(nED),       input:: real values, the equivalent dose.
+!
+! Error(nED),    input:: real values, the standard errors of equivalent dose.
+!
+! grad(npars),  output:: real values, the calculated gradient.
+!
+! value,        output:: real value, the caluculated value for the specified function.
+!
+! errorflag,    output:: integer, error message:
+!                        1.1) if both value and gradient can be calculated,errorflag=0;
+!                        1.2) if value or gradient can not be calculated, errorflag=1.
+! ====================================================================================
+! Author:: Peng Jun, 2013.04.01, revised in 2013.04.21.
+!
+! Dependence:: inner function fun34; subroutine pnorm; function alnorm.
 !
 ! Reference::  Paul Gilbert and Ravi Varadhan (2012). numDeriv: Accurate Numerical
 !              Derivatives. R package version 2012.9-1.
@@ -34,7 +44,7 @@ subroutine gradient(pars,ED,Error,npars,iter,&
   real   (kind=8),intent(out)::grad(npars)       
   real   (kind=8),intent(out)::value    
   !
-  ! local variables
+  ! Local variables
   real(kind=8),parameter::eps=1.0D-04
   real(kind=8),parameter:: d=0.0001D+00
   real(kind=8),parameter:: ZeroTol=1.781029D-05
@@ -44,13 +54,13 @@ subroutine gradient(pars,ED,Error,npars,iter,&
   real(kind=8),dimension(npars,npars)::Diag
   integer(kind=4)::i,j
   !
-  !innitializing some values
+  ! Initializing some values
   A=0.0D+00
   vtol=0.0D+00
   Diag=0.0D+00
   errorflag=0
   grad=0.0D+00
-  !calculate function's value
+  ! Calculate function's value
   ! and check for Infs and NaNs
   value=fun34(pars)
   if( value.ne.value .or. &
@@ -58,19 +68,19 @@ subroutine gradient(pars,ED,Error,npars,iter,&
     errorflag=1
     return
   end if
-  !that a diagonal matrix
+  ! That a diagonal matrix
   do i=1,npars
     Diag(i,i)=1.0D+00
   end do
-  ! special tackling with 
+  ! Special tackling with 
   ! small values in pars
   do i=1,npars
-    if(abs(pars(i))<zerotol)  vtol(i)=1.0D+00
+    if(dabs(pars(i))<zerotol)  vtol(i)=1.0D+00
   end do
-  ! set initial h value
-  h=abs(d*pars)+eps*vtol
+  ! Set initial h value
+  h=dabs(d*pars)+eps*vtol
   !
-  ! create matrix A with iter rows, npars columns.
+  ! Create matrix A with iter rows, npars columns.
   ! in each iteration, generating a new row by reduce
   ! h by 1/v.
   !.................................................
@@ -78,35 +88,35 @@ subroutine gradient(pars,ED,Error,npars,iter,&
     a(1,j)=(fun34(pars+h*Diag(j,:))-&
             fun34(pars-h*Diag(j,:)))/(2.0D+00*h(j))
   end do
-  ! check if any Inf or NaN presents in a(i,:)
+  ! Check if any Inf or NaN presents in a(i,:)
   if( any( a(1,:) .ne. a(1,:) ) .or. &
       any( a(1,:)+1.0D+00==a(1,:) )  )  then
     errorflag=1
     return
   end if
-  ! successively reduce h by 1/v.
-  h=h/real(v)
+  ! Successively reduce h by 1/v.
+  h=h/real(v,kind=8)
   !.................................................
   do i=2,iter
     do j=1,npars
-      if( i/=1 .and. abs(a(i-1,j))<1.0D-20 )  then
+      if( i/=1 .and. dabs(a(i-1,j))<1.0D-20 )  then
         a(i,j)=0.0D+00
       else 
         a(i,j)=(fun34(pars+h*Diag(j,:))-&
                 fun34(pars-h*Diag(j,:)))/(2.0D+00*h(j))
       end if
     end do
-    ! check if any Inf or NaN presents in a(i,:)
+    ! Check if any Inf or NaN presents in a(i,:)
     if( any( a(i,:) .ne. a(i,:) ) .or. &
         any( a(i,:)+1.0D+00==a(i,:) )  )  then
       errorflag=1
       return
     end if
-    ! successively reduce h by 1/v.
-    h=h/real(v)
+    ! Successively reduce h by 1/v.
+    h=h/real(v,kind=8)
   end do
   !.................................................
-  ! apply Richardson Extrapolation 
+  ! Apply Richardson Extrapolation 
   ! to improve the accuracy of gradient
   do i=1,iter-1
     a(1:(iter-i),:)=(a(2:(iter+1-i),:)*(4.0D+00**i)-&
@@ -122,7 +132,7 @@ subroutine gradient(pars,ED,Error,npars,iter,&
 !----------------------
   function fun34(x)
   !------------------------------------------------------------------------------------------------------------
-  ! fun34 is a inner function contained in subroutine hessian its used for calculating minus 
+  ! fun34() is a inner function contained in subroutine hessian its used for calculating minus 
   ! logged maximum likelihood value of minimum age model of three or four parameters 
   !
   ! x(npars), input     :: real values, the parameters used for calculating
@@ -147,21 +157,21 @@ subroutine gradient(pars,ED,Error,npars,iter,&
     !
     if (npars==3)    then
       !
-      pnorm1=(x(2)-(x(2)/x(3)**2+ED/Error**2)/(1.0D+00/x(3)**2+1.0D+00/Error**2))*sqrt(1.0D+00/Error**2+1.0D+00/x(3)**2)
+      pnorm1=(x(2)-(x(2)/x(3)**2+ED/Error**2)/(1.0D+00/x(3)**2+1.0D+00/Error**2))*dsqrt(1.0D+00/Error**2+1.0D+00/x(3)**2)
       !
       call pnorm(pnorm1,nED,upper)
       !
-      fun34=-sum(log(x(1)/sqrt(2.0D+00*pi*Error**2)*exp(-(ED-x(2))**2/(2.0D+00*Error**2))+&
-	        (1.0D+00-x(1))/sqrt(2.0D+00*pi*(Error**2+x(3)**2))*exp(-(ED-x(2))**2/(2.0D+00*(Error**2+x(3)**2)))*&
+      fun34=-sum(dlog(x(1)/dsqrt(2.0D+00*pi*Error**2)*dexp(-(ED-x(2))**2/(2.0D+00*Error**2))+&
+	        (1.0D+00-x(1))/dsqrt(2.0D+00*pi*(Error**2+x(3)**2))*dexp(-(ED-x(2))**2/(2.0D+00*(Error**2+x(3)**2)))*&
 	        (1.0D+00-pnorm1)/(0.5D+00)))  
     elseif(npars==4)  then
       !
-      pnorm1=(x(2)-(x(3)/x(4)**2+ED/Error**2)/(1.0D+00/x(4)**2+1.0D+00/Error**2))*sqrt(1.0D+00/Error**2+1.0D+00/x(4)**2)
+      pnorm1=(x(2)-(x(3)/x(4)**2+ED/Error**2)/(1.0D+00/x(4)**2+1.0D+00/Error**2))*dsqrt(1.0D+00/Error**2+1.0D+00/x(4)**2)
       !
       call pnorm(pnorm1,nED,upper)	 
       !
-      fun34=-sum(log(x(1)/sqrt(2.0D+00*pi*Error**2)*exp(-(ED-x(2))**2/(2.0D+00*Error**2))+&
-	        (1.0D+00-x(1))/sqrt(2.0D+00*pi*(Error**2+x(4)**2))*exp(-(ED-x(3))**2/(2.0D+00*(Error**2+x(4)**2)))*&
+      fun34=-sum(dlog(x(1)/dsqrt(2.0D+00*pi*Error**2)*dexp(-(ED-x(2))**2/(2.0D+00*Error**2))+&
+	        (1.0D+00-x(1))/dsqrt(2.0D+00*pi*(Error**2+x(4)**2))*dexp(-(ED-x(3))**2/(2.0D+00*(Error**2+x(4)**2)))*&
 	        (1.0D+00-pnorm1)/(1.0D+00-alnorm((x(2)-x(3))/x(4),upper))))   
       ! 
     end if
